#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------
## Daniel Wollschlger <dwoll [at] psychologie.uni-kiel.de>
## ergnzende Befehle zum Buch "Grundlagen der Datenanalyse mit R" (Springer)
## http://www.uni-kiel.de/psychologie/dwoll/r/
## Stand: 27.10.11
##
## Vgl. Abschnitt 4.1, um die folgenden Befehle auszufhren.
#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------

#####---------------------------------------------------------------------------
##### 10.9 Das allgemeine lineare Modell
##### 10.9.1 Modell der multiplen linearen Regression
##### 10.9.2 Modell der einfaktoriellen Varianzanalyse
contr.treatment(4)
options("contrasts")

contr.sum(4)
options(contrasts=c("contr.sum",       "contr.poly"))
options(contrasts=c("contr.treatment", "contr.poly"))

IV   <- gl(3, 5)
(IVe <- C(IV, contr.sum))
contrasts(IV)

DV <- rnorm(15)
lm(DV ~ IV, contrasts=list(IV=contr.treatment))

##### 10.9.3 Modell der zweifaktoriellen Varianzanalyse
##### 10.9.4 Parameterschtzungen und Vorhersage
##### 10.9.5 Parametrische Funktionen schtzen und testen
##### 10.9.6 Lineare Hypothesen als Modellvergleiche formulieren
##### 10.9.7 Modellvergleiche testen
##### 10.9.8 Beispiel: Multivariate multiple Regression
## Daten erzeugen wie in 10.5
N      <- 100
height <- rnorm(N, 175, 7)
age    <- rnorm(N, 30, 8)
sport  <- abs(rnorm(N, 60, 30))
weight <- 0.5*height - 0.3*age - 0.4*sport + 10 + rnorm(N, 0, 3)
health <- -0.3*age + 0.6*sport + rnorm(N, 4)

## fr Daten des Buches diese Datei laden:
# load("data_regrMultMV.Rdata")
Y  <- cbind(weight, health)
X  <- cbind(1, height, age, sport)
XR <- model.matrix(~ height + age + sport)
all.equal(X, XR, check.attributes=FALSE)

Xplus <- solve(t(X) %*% X) %*% t(X)
B     <- Xplus %*% Y
Pf    <- X %*% Xplus
Yhat  <- Pf %*% Y

fit <- lm(Y ~ height + age + sport)
all.equal(B,    coef(fit),   check.attributes=FALSE)
all.equal(Yhat, fitted(fit), check.attributes=FALSE)

X0  <- X[ , 1, drop=FALSE]
P0  <- X0 %*% solve(t(X0) %*% X0) %*% t(X0)

Id  <- diag(N)
WW  <- t(Y) %*% (Id - Pf) %*% Y

Xr1 <- X0
Pr1 <- P0

Xu1 <- X[ , c(1, 2)]
Pu1 <- Xu1 %*% solve(t(Xu1) %*% Xu1) %*% t(Xu1)
B1  <- t(Y) %*% (Pu1 - Pr1) %*% Y

Xr2 <- Xu1
Pr2 <- Pu1

Xu2 <- X[ , c(1, 2, 3)]
Pu2 <- Xu2 %*% solve(t(Xu2) %*% Xu2) %*% t(Xu2)
B2  <- t(Y) %*% (Pu2 - Pr2) %*% Y

Xr3 <- Xu2
Pr3 <- Pu2

Xu3 <- X
Pu3 <- Pf
B3  <- t(Y) %*% (Pu3 - Pr3) %*% Y

(WL1   <- det(WW) / det(B1 + WW))
(WL2   <- det(WW) / det(B2 + WW))
(WL3   <- det(WW) / det(B3 + WW))

(RLRl1 <- max(eigen(solve(WW) %*% B1)$values))
(RLRl2 <- max(eigen(solve(WW) %*% B2)$values))
(RLRl3 <- max(eigen(solve(WW) %*% B3)$values))

(RLRt1 <- max(eigen(solve(B1 + WW) %*% B1)$values))
(RLRt2 <- max(eigen(solve(B2 + WW) %*% B2)$values))
(RLRt3 <- max(eigen(solve(B3 + WW) %*% B3)$values))

(PBT1  <- sum(diag(solve(B1 + WW) %*% B1)))
(PBT2  <- sum(diag(solve(B2 + WW) %*% B2)))
(PBT3  <- sum(diag(solve(B3 + WW) %*% B3)))

(HLT1  <- sum(diag(solve(WW) %*% B1)))
(HLT2  <- sum(diag(solve(WW) %*% B2)))
(HLT3  <- sum(diag(solve(WW) %*% B3)))

## Kontrolle
summary(manova(fit), test="Wilks")
summary(manova(fit), test="Roy")
summary(manova(fit), test="Pillai")
summary(manova(fit), test="Hotelling-Lawley")

##### 10.9.9 Beispiel: Einfaktorielle MANOVA
## fr Daten des Buches diese Datei laden:
# load("data_manovaN.Rdata")
XstarP <- cbind(as.numeric(IVman == 1),
                as.numeric(IVman == 2),
                as.numeric(IVman == 3))

Ct   <- contr.treatment(ncol(XstarP))
Xpm1 <- XstarP %*% Ct
X    <- cbind(1, Xpm1)
XR   <- model.matrix(~ IVman)
all.equal(X, XR, check.attributes=FALSE)

Pf  <- X  %*% solve(t(X)  %*% X)  %*% t(X)
Pu  <- Pf

X0  <- X[ , 1, drop=FALSE]
P0  <- X0 %*% solve(t(X0) %*% X0) %*% t(X0)
Pr  <- P0

(Bt    <- coef(lm(Ym1 ~ IVman, contrasts=list(IVman=contr.treatment))))
splDat <- split(data.frame(Ym1), IVman)
(Mj    <- t(sapply(splDat, colMeans)))
Mj[2, ] - Mj[1, ]
Mj[3, ] - Mj[1, ]

(BstarPt <- Ct %*% Bt[-1, ])

(Be <- coef(lm(Ym1 ~ IVman, contrasts=list(IVman=contr.sum))))
colMeans(Mj)
sweep(Mj, 2, colMeans(Mj), "-")

Ce       <- contr.sum(ncol(XstarP))
(BstarPe <- Ce %*% Be[-1, ])
colSums(BstarPe)

(Bcm <- coef(lm(Ym1 ~ IVman - 1)))

Yhat <- Pf %*% Ym1
unique(Yhat)

Id <- diag(sum(Nj))
BB <- t(Ym1) %*% (Pu-Pr) %*% Ym1
WW <- t(Ym1) %*% (Id-Pf) %*% Ym1
TT <- t(Ym1) %*% (Id-P0) %*% Ym1

all.equal(BB, (sum(Nj)-1) * cov((Pu-Pr) %*% Ym1))
all.equal(WW, (sum(Nj)-1) * cov((Id-Pf) %*% Ym1))
all.equal(TT, (sum(Nj)-1) * cov((Id-P0) %*% Ym1))
all.equal(TT, (sum(Nj)-1) * cov(Ym1))
all.equal(TT, BB + WW)

(WL   <- det(WW) / det(BB + WW))
(RLRl <- max(eigen(solve(WW)      %*% BB)$values))
(RLRt <- max(eigen(solve(BB + WW) %*% BB)$values))
(PBT  <- sum( diag(solve(BB + WW) %*% BB)))
(HLT  <- sum( diag(solve(WW)      %*% BB)))

## Kontrolle
fit <- manova(Ym1 ~ IVman)
summary(fit, test="Wilks")
summary(fit, test="Roy")
summary(fit, test="Pillai")
summary(fit, test="Hotelling-Lawley")

##### 10.9.10 Beispiel: Zweifaktorielle MANOVA
## fr Daten des Buches diese Datei laden:
# load("data_manovaN.Rdata")
Xstar1  <- cbind(as.numeric(IV1 == 1),
                 as.numeric(IV1 == 2),
                 as.numeric(IV1 == 3))
Xstar2  <- cbind(as.numeric(IV2 == 1),
                 as.numeric(IV2 == 2))
Xstar12 <- cbind(as.numeric(IV1 == 1) * as.numeric(IV2 == 1),
                 as.numeric(IV1 == 2) * as.numeric(IV2 == 1),
                 as.numeric(IV1 == 3) * as.numeric(IV2 == 1),
                 as.numeric(IV1 == 1) * as.numeric(IV2 == 2),
                 as.numeric(IV1 == 2) * as.numeric(IV2 == 2),
                 as.numeric(IV1 == 3) * as.numeric(IV2 == 2))

C1  <- contr.treatment(ncol(Xstar1))
C2  <- contr.treatment(ncol(Xstar2))
C12 <- kronecker(C2, C1)

X1  <- Xstar1  %*% C1
X2  <- Xstar2  %*% C2
X12 <- Xstar12 %*% C12

X  <- cbind(1, X1, X2, X12)
XR <- model.matrix(~ IV1*IV2)
all.equal(X, XR, check.attributes=FALSE)

X0 <- X[ , 1, drop=FALSE]
P0 <- X0 %*% solve(t(X0) %*% X0) %*% t(X0)
Pf <- X  %*% solve(t(X)  %*% X)  %*% t(X)

Id <- diag(nrow(Ym2))
TT <- t(Ym2) %*% (Id - P0) %*% Ym2
WW <- t(Ym2) %*% (Id - Pf) %*% Ym2

Yhat <- Pf %*% Ym2
unique(Yhat)
splDat <- split(data.frame(Ym2), list(IV1, IV2))
t(sapply(splDat, colMeans))

Xr1 <- X0
Pr1 <- P0

Xu1 <- X[ , c(1, 2, 3)]
Pu1 <- Xu1 %*% solve(t(Xu1) %*% Xu1) %*% t(Xu1)
B1  <- t(Ym2) %*% (Pu1 - Pr1) %*% Ym2

Xr2 <- Xu1
Pr2 <- Pu1

Xu2 <- X[ , c(1, 2, 3, 4)]
Pu2 <- Xu2 %*% solve(t(Xu2) %*% Xu2) %*% t(Xu2)
B2  <- t(Ym2) %*% (Pu2 - Pr2) %*% Ym2

Xr12 <- Xu2
Pr12 <- Pu2

Xu12 <- X
Pu12 <- Pf
B12  <- t(Ym2) %*% (Pu12 - Pr12) %*% Ym2

all.equal(TT, B1 + B2 + B12 + WW)

(WL1    <- det(WW) / det(B1  + WW))
(WL2    <- det(WW) / det(B2  + WW))
(WL12   <- det(WW) / det(B12 + WW))

(RLRl1  <- max(eigen(solve(WW) %*% B1 )$values))
(RLRl2  <- max(eigen(solve(WW) %*% B2 )$values))
(RLRl12 <- max(eigen(solve(WW) %*% B12)$values))

(RLRt1  <- max(eigen(solve(B1  + WW) %*% B1 )$values))
(RLRt2  <- max(eigen(solve(B2  + WW) %*% B2 )$values))
(RLRt12 <- max(eigen(solve(B12 + WW) %*% B12)$values))

(PBT1   <- sum(diag(solve(B1  + WW) %*% B1 )))
(PBT2   <- sum(diag(solve(B2  + WW) %*% B2 )))
(PBT12  <- sum(diag(solve(B12 + WW) %*% B12)))

(HLT1   <- sum(diag(solve(WW) %*% B1 )))
(HLT2   <- sum(diag(solve(WW) %*% B2 )))
(HLT12  <- sum(diag(solve(WW) %*% B12)))

## Kontrolle
fit <- manova(Ym2 ~ IV1 * IV2)
summary(fit, test="Wilks")
summary(fit, test="Roy")
summary(fit, test="Pillai")
summary(fit, test="Hotelling-Lawley")

##### 10.9.11 Beispiel: MANCOVA
## fr Daten des Buches diese Datei laden:
# load("data_manovaN.Rdata")
IQ <- rnorm(sum(Nj), mean=100, sd=15)

Xp  <- cbind(as.numeric(IVman == 1),
             as.numeric(IVman == 2),
             as.numeric(IVman == 3))
Xpi <- cbind(as.numeric(IVman == 1) * IQ,
             as.numeric(IVman == 2) * IQ,
             as.numeric(IVman == 3) * IQ)

Ct    <- contr.treatment(ncol(Xp))
Xpm1  <- Xp  %*% Ct
Xpm1i <- Xpi %*% Ct
X     <- cbind(1, Xpm1, IQ, Xpm1i)
XR    <- model.matrix(~ IVman * IQ)
all.equal(X, XR, check.attributes=FALSE)

X0 <- X[ , 1, drop=FALSE]
P0 <- X0 %*% solve(t(X0) %*% X0) %*% t(X0)
Pf <- X  %*% solve(t(X)  %*% X)  %*% t(X)

Id <- diag(nrow(Ym1))
TT <- t(Ym1) %*% (Id - P0) %*% Ym1
WW <- t(Ym1) %*% (Id - Pf) %*% Ym1

Xr1 <- X0
Pr1 <- P0

Xu1 <- X[ , c(1, 2, 3)]
Pu1 <- Xu1 %*% solve(t(Xu1) %*% Xu1) %*% t(Xu1)
B1  <- t(Ym1) %*% (Pu1 - Pr1) %*% Ym1

Xr2 <- Xu1
Pr2 <- Pu1

Xu2 <- X[ , c(1, 2, 3, 4)]
Pu2 <- Xu2 %*% solve(t(Xu2) %*% Xu2) %*% t(Xu2)
B2  <- t(Ym1) %*% (Pu2 - Pr2) %*% Ym1

Xr12 <- Xu2
Pr12 <- Pu2

Xu12 <- X
Pu12 <- Pf
B12  <- t(Ym1) %*% (Pu12 - Pr12) %*% Ym1

all.equal(TT, B1 + B2 + B12 + WW)

(WL1    <- det(WW) / det(B1  + WW))
(WL2    <- det(WW) / det(B2  + WW))
(WL12   <- det(WW) / det(B12 + WW))

(RLRl1  <- max(eigen(solve(WW) %*% B1 )$values))
(RLRl2  <- max(eigen(solve(WW) %*% B2 )$values))
(RLRl12 <- max(eigen(solve(WW) %*% B12)$values))

(RLRt1  <- max(eigen(solve(B1  + WW) %*% B1 )$values))
(RLRt2  <- max(eigen(solve(B2  + WW) %*% B2 )$values))
(RLRt12 <- max(eigen(solve(B12 + WW) %*% B12)$values))

(PBT1   <- sum(diag(solve(B1  + WW) %*% B1 )))
(PBT2   <- sum(diag(solve(B2  + WW) %*% B2 )))
(PBT12  <- sum(diag(solve(B12 + WW) %*% B12)))

(HLT1   <- sum(diag(solve(WW) %*% B1 )))
(HLT2   <- sum(diag(solve(WW) %*% B2 )))
(HLT12  <- sum(diag(solve(WW) %*% B12)))

## Kontrolle
fit <- manova(Ym1 ~ IVman * IQ)
summary(fit, test="Wilks")
summary(fit, test="Roy")
summary(fit, test="Pillai")
summary(fit, test="Hotelling-Lawley")
