#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------
## Daniel Wollschlger <dwoll [at] psychologie.uni-kiel.de>
## ergnzende Befehle zum Buch "Grundlagen der Datenanalyse mit R" (Springer)
## http://www.uni-kiel.de/psychologie/dwoll/r/
## Stand: 10.03.11
##
## Vgl. Abschnitt 4.1, um die folgenden Befehle auszufhren.
##
## Um alle Teile des Skripts ausfhren zu knnen, muss das Paket
## boot samt Abhngigkeiten installiert sein, vgl. Abschnitt 1.2.7:
##
## install.packages("boot", repos="http://ftp5.gwdg.de/pub/misc/cran/")
#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------

##### 6.5 Kreuzvalidierung
## Daten erstellen wie in 7.3
N      <- 100
height <- rnorm(N, 175, 7)
age    <- rnorm(N,  30, 8)
sport  <- abs(rnorm(N, 60, 30))
weight <- 0.5*height - 0.3*age - 0.4*sport + 10 + rnorm(N, 0, 3)
## fr Daten des Abschnitts diese Datei laden:
# load("data_multRegrN.Rdata")

regrDf <- data.frame(weight, height, age, sport)
glmFit <- glm(weight ~ height + age + sport, data=regrDf, family=gaussian(link="identity"))

library(boot)
k    <- 3
kfCV <- cv.glm(data=regrDf, glmfit=glmFit, K=k)
kfCV$delta

doCV <- function(idxTst) {
    fitTrn  <- lm(weight ~ height + age + sport, regrDf, subset=!idxTst)
    predTst <- predict(fitTrn, regrDf[idxTst, ])
    mean((predTst - weight[idxTst])^2)
}

tstGrp <- (sample(1:N, N, replace=FALSE) %% k) + 1
## fr Daten des Abschnitts diese Datei laden:
# load("data_multRegrN.Rdata")
(tst1  <- doCV(tstGrp == 1))
(tst2  <- doCV(tstGrp == 2))
(tst3  <- doCV(tstGrp == 3))
mean(c(tst1, tst2, tst3))

LOOCV <- cv.glm(data=regrDf, glmfit=glmFit, K=N)
LOOCV$delta

lmFit <- lm(weight ~ height + age + sport)
PRESS <- (residuals(lmFit) / (1 - hatvalues(lmFit)))^2
mean(PRESS)

idx <- seq(along=numeric(N))
res <- sapply(idx, function(x) { doCV(idx == x) } )
mean(res)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern mglich
try(detach(package:boot))
