#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------
## Daniel Wollschlger <dwoll [at] psychologie.uni-kiel.de>
## ergnzende Befehle zum Buch "Grundlagen der Datenanalyse mit R" (Springer)
## http://www.uni-kiel.de/psychologie/dwoll/r/
## Stand: 29.04.11
##
## Vgl. Abschnitt 4.1, um die folgenden Befehle auszufhren.
##
## Um alle Teile des Skripts ausfhren zu knnen, muss das Paket
## car samt Abhngigkeiten installiert sein, vgl. Abschnitt 1.2.7:
##
## install.packages("car", repos="http://ftp5.gwdg.de/pub/misc/cran/")
#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------

##### 6.6 Regressionsdiagnostik
##### 6.6.1
## Daten erstellen wie in 6.3
N      <- 100
height <- rnorm(N, 175, 7)
age    <- rnorm(N,  30, 8)
sport  <- abs(rnorm(N, 60, 30))
weight <- 0.5*height - 0.3*age - 0.4*sport + 10 + rnorm(N, 0, 3)
## fr Daten des Abschnitts diese Datei laden:
# load("data_multRegrN.Rdata")

Xpred <- cbind(height, age, sport)

dev.new(width=12, height=4.5)
op <- par(mfrow=c(1, 3), cex.lab=1.5, cex.main=1.5)
boxplot(Xpred, main="Verteilung der Prdiktoren")

zX <- scale(Xpred)
summary(zX)

ctrX   <- colMeans(Xpred)
sX     <- cov(Xpred)
mahaSq <- mahalanobis(Xpred, ctrX, sX)
summary(sqrt(mahaSq))

fitHAS <- lm(weight ~ height + age + sport)
h      <- hatvalues(fitHAS)
hist(h, main="Histogramm der Hebelwerte")
summary(h)

all.equal(mahaSq, (N-1)*(h-(1/N)))

(inflRes <- influence.measures(fitHAS))
infMat   <- inflRes$is.inf
infMat[rowSums(infMat) > 0, ]

cooksDst <- cooks.distance(fitHAS)
plot(cooksDst, main="Cooks Distanz", type="h")
par(op)

P   <- 3
E   <- residuals(fitHAS)
MSE <- sum(E^2) / (N - (P+1))
CD  <- (E^2 / (MSE * (1-h)^2)) * (h / (P+1))
all.equal(cooksDst, CD)

##### 6.6.2
## fr Daten des Abschnitts diese Datei laden:
# load("data_multRegrN.Rdata")
Estnd <- rstandard(fitHAS)
all.equal(Estnd, E / sqrt(MSE * (1-h)))

Estud <- rstudent(fitHAS)
all.equal(Estud, E / (lm.influence(fitHAS)$sigma * sqrt(1-h)))

dev.new(width=12, height=4.5)
op <- par(mfrow=c(1, 3), cex.lab=1.5, cex.main=1.5)
hist(Estud, main="Histogramm stud. Residuen", freq=FALSE)
curve(dnorm(x, mean=0, sd=1), col="red", lwd=2, add=TRUE)

qqnorm(Estud, main="Q-Q-Plot stud. Residuen")
qqline(Estud, col="red", lwd=2)

shapiro.test(Estud)

plot(fitted(fitHAS), Estud, pch=20, ylab="studentisierte Residuen",
     xlab="Vorhersage", main="Spread-Level-Plot")
abline(h=0, col="red", lwd=2)
par(op)

library(car)
bcRes   <- boxCox(fitHAS)
str(bcRes)
(lambda <- bcRes$x[which.max(bcRes$y)])
wTrans  <- bcPower(weight, lambda)
all.equal(wTrans, ((weight^lambda)-1) / lambda)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern mglich
try({
detach(package:car)
detach(package:survival)
detach(package:splines)
detach(package:nnet)
detach(package:MASS) })

##### 6.6.3
## fr Daten des Abschnitts diese Datei laden:
# load("data_multRegrN.Rdata")
(Rx <- cor(cbind(height, age, sport)))

library(car)
vif(fitHAS)

fitHeight <- lm(height ~ age    + sport)
fitAge    <- lm(age    ~ height + sport)
fitSport  <- lm(sport  ~ height + age)

1 / (1-summary(fitHeight)$r.squared)
1 / (1-summary(fitAge   )$r.squared)
1 / (1-summary(fitSport )$r.squared)

diag(solve(Rx))

fitScl <- lm(scale(weight) ~ scale(height) + scale(age) + scale(sport))
X      <- model.matrix(fitScl)
kappa(fitScl, exact=TRUE)
(eigVals <- eigen(t(X) %*% X)$values)
sqrt(eigVals / min(eigVals[eigVals >= .Machine$double.eps]))

vif(fitScl)
kappa(lm(weight ~ height + age + sport), exact=TRUE)

##### 6.6.4

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern mglich
try({
detach(package:car)
detach(package:survival)
detach(package:splines)
detach(package:nnet)
detach(package:MASS) })

