#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------
## Daniel Wollschlger <dwoll [at] psychologie.uni-kiel.de>
## ergnzende Befehle zum Buch "Grundlagen der Datenanalyse mit R" (Springer)
## http://www.uni-kiel.de/psychologie/dwoll/r/
## Stand: 31.08.10
##
## Vgl. Abschnitt 4.1, um die folgenden Befehle auszufhren.
#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------

##### 7.5.3 Test bedingter Haupteffekte im CRF-pq Design
## Daten erstellen wie in 7.5.1
Njk <- 8
P   <- 2
Q   <- 3
IV1 <- factor(rep(1:P, Njk*Q))
IV2 <- factor(rep(1:Q, each=Njk*P))
DV  <- rnorm(Njk*P*Q, 0, 1)
DV[IV1 == 1] <- DV[IV1 == 1] + 0.5
DV[IV1 == 2] <- DV[IV1 == 2] - 0.5
DV[IV2 == 1] <- DV[IV2 == 1] - 0.5
DV[IV2 == 3] <- DV[IV2 == 3] + 0.5
dfCRFpq <- data.frame(IV1, IV2, DV)
## fr Daten des Abschnitts diese Datei laden:
# load("data_crfpq.Rdata")

CRFp1 <- anova(lm(DV ~ IV1, data=dfCRFpq, subset=(IV2==1)))
CRFp2 <- anova(lm(DV ~ IV1, data=dfCRFpq, subset=(IV2==2)))
CRFp3 <- anova(lm(DV ~ IV1, data=dfCRFpq, subset=(IV2==3)))

SSp1  <- CRFp1[["Sum Sq"]][1]
SSp2  <- CRFp2[["Sum Sq"]][1]
SSp3  <- CRFp3[["Sum Sq"]][1]

CRFpq <- anova(lm(DV ~ IV1*IV2, data=dfCRFpq))
SSA   <- CRFpq[["Sum Sq"]][1]
SSI   <- CRFpq[["Sum Sq"]][3]
SSE   <- CRFpq[["Sum Sq"]][4]
dfSSA <- CRFpq[["Df"]][1]
dfSSE <- CRFpq[["Df"]][4]

all.equal(SSp1 + SSp2 + SSp3, SSA + SSI)

Fp1   <- (SSp1/dfSSA) / (SSE/dfSSE)
Fp2   <- (SSp2/dfSSA) / (SSE/dfSSE)
Fp3   <- (SSp3/dfSSA) / (SSE/dfSSE)

(pP1  <- 1-pf(Fp1, dfSSA, dfSSE))
(pP2  <- 1-pf(Fp2, dfSSA, dfSSE))
(pP3  <- 1-pf(Fp3, dfSSA, dfSSE))
